package com.sds.portal.file.command;

import com.oreilly.servlet.ParameterParser;

import java.io.*;
import javax.servlet.http.*;
import javax.naming.*;
import javax.servlet.*;

import com.sds.portal.file.delegate.FileDelegate;
import com.sds.portal.file.vo.*;
import com.sds.portal.file.command.*;
import com.sds.common.command.Command;
import com.sds.common.util.*;

public class DownloadFileCmd extends Command {
    private String forwardPage;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1 
     */
    private FileDelegate fileDelegate;
    /**
     * @clientCardinality 1
     * @supplierCardinality 1 
     */
    private DiskFileHelper diskFileHelper = null;

    public DownloadFileCmd(String forwardPage) {
        super(forwardPage);
    }

    public DownloadFileCmd() {
        super();
    }

    public String execute(HttpServletRequest req, HttpServletResponse res) throws Exception {
        Context ctx = null;
        Locator locator = Locator.getInstance();

		// parameter data
		Integer fileId = null;
		String repositoryPath = null;
		String aliasFileName = null;
		String originalFileName = null;
        Long fileSize = null;

		InputStream inputStream = null;
		OutputStream outputStream = null;
		
		// determin interface type
        Boolean isLocalInteface = (Boolean) locator.lookup("java:comp/env/file/isLocalInterface");

		// delegate
		fileDelegate = new FileDelegate(isLocalInteface);

		try {
            // request parser
            ParameterParser parser = new ParameterParser(req);
            
            // execute bisiness logic
            fileId = new Integer(parser.getIntParameter("fileId"));
            FileVo fileVo = fileDelegate.getFileVo(fileId);

            diskFileHelper = new DiskFileHelper();
            diskFileHelper.download(req, res, fileVo);

            // increase download count
            fileDelegate.increaseDownloads(fileId);

            return this.getForwardPage();
            
        } catch (FileNotFoundException ex) {
            // if the file does not exist, is a directory rather than a regular file, or for some other reason cannot be opened for reading.
            throw ex;
        } catch (SecurityException ex) {
            // if a security manager exists and its checkRead method denies read access to the file.
            throw ex;
        } catch (IOException ex) {
            // if an file I/O error occurs.
            throw ex;
        } catch (Exception ex) {
            throw ex;
        } finally {
            if (ctx != null) ctx.close();
            if (inputStream != null) inputStream.close();
            if (outputStream != null) outputStream.close();
        }
    }
}
