package com.sds.portal.post.command;

import com.oreilly.servlet.ParameterParser;

import java.util.*;
import java.io.*;
import javax.servlet.http.*;
import javax.servlet.*;
import javax.naming.*;
import javax.transaction.*;

import com.sds.common.command.*;
import com.sds.portal.post.delegate.QNAFileDelegate;
import com.sds.portal.file.delegate.FileDelegate;
import com.sds.portal.file.vo.*;
import com.sds.common.util.*;

import org.apache.commons.fileupload.*;

public class DeleteFileCmd extends Command {
    private String forwardPage;

    /**
     * @supplierCardinality 1
     * @clientCardinality 1 
     */
    private QNAFileDelegate qnaFileDelegate;
    /**
     * @supplierCardinality 1
     * @clientCardinality 1 
     */
    private FileDelegate fileDelegate;
    public final static String PARAM_FILE_ID = "fileId";

    public String execute(HttpServletRequest req, HttpServletResponse res) throws Exception {
		Context ctx = null;
        Locator locator = Locator.getInstance();

		// parameter data
		Integer fileId = null;
		String repositoryPath = null;
		String aliasFileName = null;
		String originalFileName = null;

		InputStream inputStream = null;
		OutputStream outputStream = null;

		// determin interface type
        Boolean isLocalIntefaceQNAFile = (Boolean) locator.lookup("java:comp/env/qnaFile/isLocalInterface");
        Boolean isLocalIntefaceFile = (Boolean) locator.lookup("java:comp/env/file/isLocalInterface");

		// delegate
		qnaFileDelegate = new QNAFileDelegate(isLocalIntefaceQNAFile);
        fileDelegate = new FileDelegate(isLocalIntefaceFile);

		try {
            // request parser
            ParameterParser parser = new ParameterParser(req);
            DiskFileUpload diskFileUpload = new DiskFileUpload();
            List items = diskFileUpload.parseRequest(req);

            String parameterName = null;
            String parameterValue = null;

            for(Iterator it = items.iterator(); it.hasNext();) {
                FileItem item = (FileItem)it.next();
                if(item.isFormField()) {
                    parameterName = item.getFieldName();
                    parameterValue = item.getString();
                    System.out.println("parameter name: " + parameterName + ", value: " + parameterValue);
                    if(parameterName.equals(this.PARAM_FILE_ID) && parameterValue != null)
                        fileId = new Integer(parameterValue);
                }
            }

            FileVo fileVo = fileDelegate.getFileVo(fileId);
            repositoryPath = fileVo.getRepositoryPath();
            aliasFileName = fileVo.getAliasFileName();

            if (fileId == null) throw new Exception();
            if (repositoryPath == null) throw new Exception();
            if (aliasFileName == null) throw new Exception();

            // delete file on filesystem
            File file = new File(repositoryPath + aliasFileName);
            file.delete();

            // execute adapter
            try {
				String[] adapteeClassNames = (String[]) locator.lookup("java:comp/env/qnaFile/deleteFileCmdAdapteeClassNames", ",");
				Adapter adapter = new Adapter(req, res);
				adapter.executeAdapter(adapteeClassNames);
            } catch (NameNotFoundException ex) {
            } catch (Exception ex) {
                throw new Exception();
            }

            // delete file info on database
            qnaFileDelegate.deleteFile(fileId);
            return this.getForwardPage();
        } catch (Exception ex) {
            ex.printStackTrace();
            throw ex;
        } finally {
            if (inputStream != null) inputStream.close();
            if (outputStream != null) outputStream.close();
        }
    }

    public DeleteFileCmd(String forwardPage, boolean isDuplicateAllowed) {
        super(forwardPage, isDuplicateAllowed);
    }

    public DeleteFileCmd(String forwardPage) {
        super(forwardPage);
    }
    public DeleteFileCmd() {
        super();
    }
}
