package com.sds.portal.file.command;

import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import javax.naming.*;
import javax.transaction.*;

import org.w3c.util.UUID;
import org.apache.commons.fileupload.*;
import com.oreilly.servlet.*;

import com.sds.portal.file.delegate.*;
import com.sds.portal.file.vo.*;
import com.sds.common.util.*;
import com.sds.common.command.Command;

public class DiskFileHelper {
    private String repositoryPath = null;
	private int bufferSize = 0;
    private List fileItems;
    private Vector fileNames;
    private static final int BUFFER_SIZE =  256;
    private Locator locator = null;

	public DiskFileHelper() throws Exception {
        Context ctx = null;
        this.fileNames = new Vector();
        try {
            locator = Locator.getInstance();
            repositoryPath = (String) locator.lookup("java:comp/env/file/repositoryPath");
            try {
	            bufferSize = ((Integer) Locator.getInstance().lookup("java:comp/env/file/bufferSize")).intValue();
            } catch (Exception ex) {
                bufferSize = this.BUFFER_SIZE;
            }
        } catch (Exception ex) {
            throw ex;
        } finally {
            if (ctx != null) ctx.close();
        }
    }

    public void delete() throws Exception {
        delete(this.fileNames);
    }

    public FileVo upload(FileItem item) throws Exception {
        if (item.isFormField()) throw new IllegalArgumentException();
		String originalFileName = null;
		String aliasFileName = null;
		Long fileSize = null;

        originalFileName = item.getName();
		// for cross-browser
		// ie: originalFileName = full path name + file name
		// ns: originalFileName = file name
		int lastIndex = originalFileName.lastIndexOf(File.separator);
		originalFileName = originalFileName.substring(lastIndex + 1);

		// skip empty upload file
		if (originalFileName == null || originalFileName.length() == 0) return null;
		aliasFileName = (new UUID()).toString();
		fileSize = new Long(item.getSize());

		// save file
        Locator locator = Locator.getInstance();
        String repositoryPath = (String) locator.lookup("java:comp/env/file/repositoryPath");
		String uploadFileName = repositoryPath + aliasFileName;
		File file = new File(uploadFileName);
        this.fileNames.add(uploadFileName);

if (file == null) System.out.println("file is null");
if (item == null) System.out.println("item is null");

		item.write(file);

		// set vo
		FileVo fileVo = new FileVo();
		fileVo.setOriginalFileName(originalFileName);
		fileVo.setAliasFileName(aliasFileName);
		fileVo.setRepositoryPath(repositoryPath);
		fileVo.setFileSize(fileSize);
		fileVo.setDownloads(new Integer(0));

		return fileVo;
    }

    public void delete(Vector fileNames) throws Exception {
        Iterator iter = fileNames.iterator();
        if (iter.hasNext()) {
            this.delete((String) iter.next());
        }
    }

    public void delete(String fileName) throws Exception {
        File file = new File(fileName);
        file.delete();
    }

    public FileVo[] upload(List fileItems) throws Exception {
		try {
            Iterator iter = fileItems.iterator();
            Vector vector = new Vector();
            FileVo fileVo = null;
            while (iter.hasNext()) {
                FileItem item = (FileItem) iter.next();
                if (!item.isFormField()) {
	                try {
		                fileVo = this.upload(item);
        	        } catch (Exception ex) {}
            	   	if (fileVo != null) vector.add(fileVo);
               	}
            }
            return (FileVo[]) vector.toArray(new FileVo[vector.size()]);
        } catch (Exception ex) {
            ex.printStackTrace();
            this.delete();
            throw ex;
        }
    }

    public List getFileItems(){ return fileItems; }
    public void setFileItems(List fileItems){ this.fileItems = fileItems; }

    public Vector getFileNames() { return fileNames; }

    public void setFileNames(Vector fileNames) { this.fileNames = fileNames; }

    public void download(HttpServletRequest req, HttpServletResponse res, FileVo fileVo) throws Exception {
        Integer fileId = null;
		String aliasFileName = null;
		String originalFileName = null;
        Long fileSize = null;

        InputStream inputStream = null;
		OutputStream outputStream = null;

		fileId = fileVo.getFileId();
        repositoryPath = fileVo.getRepositoryPath();
		aliasFileName = fileVo.getAliasFileName();
		originalFileName = fileVo.getOriginalFileName();
		fileSize = fileVo.getFileSize();

		if (fileId == null) throw new Exception();
		if (repositoryPath == null) throw new Exception();
		if (aliasFileName == null) throw new Exception();
		if (originalFileName == null) throw new Exception();
		if (fileSize == null) throw new Exception();

		// send file
		File downloadFile = new File(repositoryPath + aliasFileName);
		res.setHeader("Content-Type", "");            
		if (req.getHeader("User-Agent").indexOf("MSIE 5.5") > -1)
			res.setHeader("Content-Disposition", "filename="+originalFileName);
		else     
			res.setHeader("Content-Disposition", "attachment; filename="+originalFileName);
		inputStream = new FileInputStream(downloadFile);
		outputStream = res.getOutputStream();

		// set buffer size
		byte buffer[] = new byte[bufferSize];

		int readLength = 0;
		long totalLength = 0;
		while ((readLength = inputStream.read(buffer)) > 0) {
			totalLength += readLength;
			outputStream.write(buffer, 0, readLength);
		}
    }
}
