package com.sds.portal.post.command;

import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import javax.naming.*;
import javax.transaction.*;

import org.apache.commons.fileupload.*;

import com.sds.portal.post.delegate.QNAFileDelegate;
import com.sds.portal.post.vo.CompleteQNAVo;
import com.sds.portal.file.command.DiskFileHelper;
import com.sds.portal.file.EncTypeException;
import com.sds.portal.file.MethodException;
import com.sds.common.command.*;
import com.sds.common.util.*;
import com.sds.portal.file.vo.*;
import com.sds.portal.post.vo.*;

public class PostQNAFileCmd extends Command {
    public final static String PARAM_BOARD_ID = "boardId";
    public final static String PARAM_AUTHOR_ID = "authorId";
    public final static String PARAM_TITLE = "title";
    public final static String PARAM_CONTENT = "content";
    public final static String PARAM_HTML_FLAG = "htmlFlag";
    public final static String PARAM_PARENT_POST_ID = "parentPostId";
    
    private String forwardPage;


    /**
     * @clientCardinality 1
     * @supplierCardinality 1 
     */
    private QNAFileDelegate qnaFileDelegate;
    /**
     * @clientCardinality 1
     * @supplierCardinality 1 
     */
    private DiskFileHelper diskFileHelper = null;

    /** @link dependency 
     * @stereotype use*/
    /*# CompleteQNAVo lnkCompleteQNAVo; */

    public PostQNAFileCmd(String forwardPage, boolean isDuplicateAllowed) {
        super(forwardPage, isDuplicateAllowed);
    }

    public PostQNAFileCmd(String forwardPage) {
        super(forwardPage);
    }
    public PostQNAFileCmd() {
    }

    public String execute(HttpServletRequest req, HttpServletResponse res) throws Exception {
        diskFileHelper = new DiskFileHelper();
        Locator locator = Locator.getInstance();

        CompleteQNAVo qnaVo = null;

		// paremeter data
		Integer boardId = null;
		Integer parentPostId = null;
		String title = null;
		String content = null;
		String authorId = null;
        Boolean htmlFlag = null;
        String ipAddr = null;

        // return value
        Integer postId = null;

		// determin interface type
        Boolean isLocalInteface = (Boolean) locator.lookup("java:comp/env/qnaFile/isLocalInterface");

		// delegate
		qnaFileDelegate = new QNAFileDelegate(isLocalInteface);

        String parameterName = null;
        String parameterValue = null;
        String[] parameterValues = null;

		try {
            boolean isMultiPart = FileUpload.isMultipartContent(req);
            if (!isMultiPart) throw new EncTypeException("enctype should be 'multipart/form-data'");
            if (!req.getMethod().equals("POST")) throw new MethodException("method should be 'post'");

            DiskFileUpload diskFileUpload = new DiskFileUpload();
            List items = diskFileUpload.parseRequest(req);
            if (items.isEmpty()) throw new Exception("FileItems are empty");

            diskFileHelper = new DiskFileHelper();
            FileVo[] fileVoArray = null;
            Vector vector = new Vector();
            
            Iterator it = items.iterator();
            while (it.hasNext()) {
                FileVo tmpFileVo = null;
                FileItem item = (FileItem) it.next();
                if (!item.isFormField()) {
                    tmpFileVo = diskFileHelper.upload(item);
                    if (tmpFileVo != null) vector.add(tmpFileVo);
                } else {
                    parameterName = item.getFieldName();
                    parameterValue = item.getString();

                    // debuf info
                    System.out.println("parameter name: "+ parameterName + ", value: "+ parameterValue);
                    
                    if (parameterName.equals(this.PARAM_PARENT_POST_ID) && parameterValue != null) parentPostId = new Integer(parameterValue);
                    if (parameterName.equals(this.PARAM_BOARD_ID) && parameterValue != null) boardId = new Integer(parameterValue);
                    if (parameterName.equals(this.PARAM_TITLE)) title = parameterValue;
                    if (parameterName.equals(this.PARAM_CONTENT)) content = parameterValue;
                    if (parameterName.equals(this.PARAM_HTML_FLAG) && parameterValue != null) htmlFlag = new Boolean(parameterValue);
                    ipAddr = req.getRemoteAddr();
                    if (parameterName.equals(this.PARAM_AUTHOR_ID)) authorId = parameterValue;
                }
            }
            fileVoArray = (FileVo[]) vector.toArray(new FileVo[vector.size()]);

            // set vo
            qnaVo = new CompleteQNAVo();
            qnaVo.setBoardId(boardId);
            qnaVo.setParentPostId(parentPostId);
            qnaVo.setTitle(title);
            qnaVo.setContent(content);
            qnaVo.setIpAddr(ipAddr);
            qnaVo.setAuthorId(authorId);
            qnaVo.setViews(new Integer(0));
            qnaVo.setMaskFlag(Boolean.FALSE);
            qnaVo.setHtmlFlag(htmlFlag);

            // execute adapter
            try {
				String[] adapteeClassNames = (String[]) locator.lookup("java:comp/env/qnaFile/postQNACmdAdapteeClassNames", ",");
				Adapter adapter = new Adapter(req, res);
				adapter.executeAdapter(adapteeClassNames);
            } catch (NameNotFoundException ex) {
            } catch (Exception ex) {
                throw new Exception();
            }

            // post qnaFileVo
            CompleteQNAFileVo qnaFileVo = new CompleteQNAFileVo();
            qnaFileVo.setQNAVo(qnaVo);
            qnaFileVo.setFileVoArray(fileVoArray);
            qnaFileDelegate.post(qnaFileVo);

            return this.getForwardPage();
        } catch (Exception ex) {
            ex.printStackTrace();
            diskFileHelper.delete();
            qnaFileDelegate.deleteQNA(postId);
            throw ex;
        }
    }
}
